from __future__ import annotations

from .en.STS12VisualSTS import *
from .en.STS13VisualSTS import *
from .en.STS14VisualSTS import *
from .en.STS15VisualSTS import *
from .en.STS16VisualSTS import *
from .multilingual.STS17MultilingualVisualSTS import *
from .multilingual.STSBenchmarkMultilingualVisualSTS import *
